﻿using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class GridManager : MonoBehaviour
{
    public static GridManager Instance;
    public event Action<GridTile> OnStreetTileClicked;
    public event Action<GridTile> OnBuildingTileClicked;
    public event Action<GridTile> OnCrimeStarted;

    public List<GridTile> tiles;
    public List<GridTile> streetTiles;
    public List<GridTile> buildingTiles;
    public List<GridTile> monitoredTiled;

    public bool showCrimeRates = false;
    private void Start ()
    {
        Instance = this;
        streetTiles = new List<GridTile>();
        buildingTiles = new List<GridTile>();
        monitoredTiled = new List<GridTile>();
	    foreach(GridTile __tile in tiles)
        {
            __tile.OnMouseClick += TileClicked;
            __tile.OnCrimeStarted += CrimeStarted;
            if (__tile.tileType == GridTile.TileType.STREET)
                streetTiles.Add(__tile);
            else
                buildingTiles.Add(__tile);
        }
	}

    private void CrimeStarted(GridTile p_tile)
    {
        if (OnCrimeStarted != null)
            OnCrimeStarted(p_tile);
    }

    private void TileClicked(GridTile p_tile)
    {
        if (p_tile.tileType == GridTile.TileType.STREET && OnStreetTileClicked != null)
            OnStreetTileClicked(p_tile);
        else if (p_tile.tileType == GridTile.TileType.BUILDING && OnBuildingTileClicked != null)
            OnBuildingTileClicked(p_tile);
    }

    private void Update()
    {
        /*if (Input.GetKeyDown(KeyCode.F4))
        {
            foreach (GridTile __tile in streetTiles)
                __tile.EnableCrimeRateIndicator(true, true);
        }
        else if (Input.GetKeyUp(KeyCode.F4))
        {
            foreach (GridTile __tile in streetTiles)
                __tile.EnableCrimeRateIndicator(false,true);
        }*/

        if (showCrimeRates)
        {
            foreach (GridTile __tile in streetTiles)
            {
                __tile.EnableCrimeRateIndicator(true, false);
            }
        }
    }
    public void MouseEnterCrimeRates()
    {
        showCrimeRates = true;
    }
    public void MouseExitCrimeRates()
    {
        showCrimeRates = false;
        if (!Input.GetKey(KeyCode.F1))
        {
            foreach (GridTile __tile in streetTiles)
            {
                __tile.EnableCrimeRateIndicator(false, false);
            }
        }
    }

    public void UpdateMonitoredTiles(List<PoliceUnit> p_units, List<PoliceCamera> p_cameras, List<PoliceStation> p_stations)
    {
        monitoredTiled = new List<GridTile>();
        foreach (PoliceUnit __unit in p_units)
            AddTilesToMonitoredList(__unit.monitoredTiles);
        foreach (PoliceCamera __camera in p_cameras)
            AddTilesToMonitoredList(__camera.monitoredTiles);
        foreach (PoliceStation __station in p_stations)
            AddTilesToMonitoredList(__station.monitoredTiles);
    }
    private void AddTilesToMonitoredList(List<GridTile> p_tiles)
    {
        foreach (GridTile __tile in p_tiles)
            if (__tile != null && !monitoredTiled.Contains(__tile))
                monitoredTiled.Add(__tile);
    }
    public GridTile GetTileByIndex(int p_indexX, int p_indexY)
    {
        if (p_indexX < 0 || p_indexX > 13 || p_indexY < 0 || p_indexY > 13)
            return null;
        foreach (GridTile __tile in tiles)
            if (__tile.tileX == p_indexX && __tile.tileY == p_indexY)
                return __tile;
        return null;
    }
}
