﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PoliceCamera : MonoBehaviour
{
    public Orientation          orientation = Orientation.LEFT;
    public List<SpriteRenderer> areasOfEffect;
    public List<GridTile>       monitoredTiles;
    public GridTile             tile;

    public SpriteRenderer       sprite;
    private void Start()
    {
        UpdateAoEs();
    }
    private void OnMouseDown()
    {
        if (GameSceneManager.GameState == GameSceneManager.GameStateType.NORMAL)
        {
            GameSceneManager.Instance.selectedCamera = this;
            GameSceneManager.Instance.ChangeToEditingStateState(GameSceneManager.GameStateType.EDITING_POLICE_CAMERA);
            UIManager.Instance.UpdateSellButtonLabel(0);
        }
    }
    public void RotateCamera()
    {
        transform.Rotate(Vector3.forward * -90f);
        orientation++;
        if ((int)orientation == 4)
            orientation = Orientation.UP;
        if (orientation == Orientation.RIGHT)
        {
            sprite.flipX = true;
            sprite.flipY = true;
        }
        else 
        {
            sprite.flipX = true;
            sprite.flipY = false;
        }
        UpdateAoEs();
    }
    private void UpdateAoEs()
    {
        monitoredTiles = new List<GridTile>();
        monitoredTiles.Add(tile);
        if (orientation == Orientation.UP)
        {
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX, tile.tileY - 1));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX, tile.tileY - 2));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX, tile.tileY - 3));
        }
        else if (orientation == Orientation.RIGHT)
        {
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX + 1, tile.tileY));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX + 2, tile.tileY));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX + 3, tile.tileY));
        }
        else if (orientation == Orientation.DOWN)
        {
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX, tile.tileY + 1));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX, tile.tileY + 2));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX, tile.tileY + 3));
        }
        else if (orientation == Orientation.LEFT)
        {
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX - 1, tile.tileY));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX - 2, tile.tileY));
            monitoredTiles.Add(GridManager.Instance.GetTileByIndex(tile.tileX - 3, tile.tileY));
        }
        for (int i = 0; i <= 3; i++)
        {
            if (monitoredTiles[i] == null || monitoredTiles[i].tileType == GridTile.TileType.BUILDING)
            {
                monitoredTiles.RemoveRange(i, 4 - i);
                break;
            }
        }
        GameSceneManager.RequestUpdateMonitoredTiles();
        EnableAoEs(monitoredTiles.Count - 1);
    }
    private void EnableAoEs(int p_enableCount)
    {
        areasOfEffect[0].gameObject.SetActive(p_enableCount > 0 ? true: false);
        areasOfEffect[1].gameObject.SetActive(p_enableCount > 1 ? true : false);
        areasOfEffect[2].gameObject.SetActive(p_enableCount > 2 ? true : false);
    }
}
