﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PoliceUnit : MonoBehaviour
{
    public List<GridTile> route;
    public List<Orientation> routeOrientations;
    public List<GridTile> monitoredTiles;
    
    private int routeIndex = 0;
    private float routeCounter;
    
    public float speed = 2f;
    public int viewRange;
    public Animator animator;
    public bool isPoliceCar = false;

    private void Start()
    {
        UpdateMonitoredTiles();
    }
    private void Update()
    {
        routeCounter += Time.deltaTime * speed;
        if (routeCounter >= 1f)
        {
            routeCounter -= 1f;
            routeIndex++;
            if (routeIndex == route.Count)
                routeIndex = 0;
            UpdateMonitoredTiles();
            animator.SetInteger("Orientation", (int)routeOrientations[routeIndex]);
        }
        UpdatePoliceManPosition();
    }
    private void OnMouseDown()
    {
        if (GameSceneManager.GameState == GameSceneManager.GameStateType.NORMAL)
        {
            GameSceneManager.Instance.selectedUnit = this;
            if (isPoliceCar)
            {
                GameSceneManager.Instance.ChangeToEditingStateState(GameSceneManager.GameStateType.EDITING_POLICE_CAR);
                UIManager.Instance.UpdateSellButtonLabel(3);
            }
            else
            {
                GameSceneManager.Instance.ChangeToEditingStateState(GameSceneManager.GameStateType.EDITING_POLICE_MAN);
                UIManager.Instance.UpdateSellButtonLabel(1);
            }
        }
    }
    private void UpdatePoliceManPosition()
    {
        transform.position = Vector3.Lerp(route[routeIndex].transform.position, GetNextTilePosition(), routeCounter);
        transform.position -= Vector3.forward;
    }
    private void UpdateMonitoredTiles()
    {
        monitoredTiles = new List<GridTile>();
        GridTile __currentTile = route[routeIndex];
        //Row
        for (int i = __currentTile.tileY - viewRange; i <= __currentTile.tileY + viewRange; i++)
        {
            //Column
            for (int j = __currentTile.tileX - viewRange; j <= __currentTile.tileX + viewRange; j++)
            {
                monitoredTiles.Add(GridManager.Instance.GetTileByIndex(j, i));
            }
        }
        GameSceneManager.RequestUpdateMonitoredTiles();
    }
    private Vector3 GetNextTilePosition()
    {
        if (routeIndex + 1 == route.Count)
            return route[0].transform.position;
        else
            return route[routeIndex + 1].transform.position;
    }
}
